﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GraphBundling2D
{
    class Database
    {
        #region Attributes
        public Vertex[][] CurrentTraj;
        public Vertex[][] originalTraj;
        #endregion

        #region Constructor
        public Database()
        {
            originalTraj = CreateTestDataset();
            CurrentTraj = CloneDataBase(originalTraj);
        }
        #endregion

        #region CloneDataBase
        private Vertex[][] CloneDataBase(Vertex[][] RecordsOriginal)
        {
            Vertex[][] clone = new Vertex[RecordsOriginal.Length][];
            for (int i = 0; i < RecordsOriginal.Length; i++)
            {
                clone[i] = new Vertex[RecordsOriginal[i].Length];

                for (int j = 0; j < RecordsOriginal[i].Length; j++)
                {
                    clone[i][j] = Vertex.CloneVertex(RecordsOriginal[i][j]);
                }
            }
            return clone;
        }
        #endregion

        #region CreateTestDataset
        private Vertex[][] CreateTestDataset()
        {
            int nbrLines = 9;
            int nbrPointsPerLine = 9;

            Vertex[][] rec = new Vertex[nbrLines][];

            for (int i = 0; i < nbrLines; i++)
            {
                rec[i] = new Vertex[nbrPointsPerLine];

                for (int j = 0; j < nbrPointsPerLine; j++)
                {
                    double incy = 1.0 / nbrPointsPerLine;
                    double incx = (1.0 - 2 * (double)(i + 0.5) / (double)nbrLines) / (double)nbrPointsPerLine;
                    rec[i][j] = new Vertex((double)i / (double)nbrLines + incx * (double)(j + 0.5), incy * (double)(j + 0.5), 1.0f);
                }
            }

            return rec;
        }
        #endregion

        #region Getters
        public Vertex[][] getOriginalTraj()
        {
            return this.originalTraj;
        }

        public Vertex[][] getCurrentTraj()
        {
            return this.CurrentTraj;
        }
        #endregion

        #region UpdateDatabase
        public void UpdateDatabase(Vertex[][] newVertices)
        {
            originalTraj = CloneDataBase(newVertices);
            CurrentTraj = CloneDataBase(originalTraj);
        }
        #endregion
    }
}

