﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using KDEBundling;
using OpenTK;
using OpenTK.Graphics.OpenGL;

namespace BundlingExample
{
    public partial class KDEBundlingForm : Form
    {
        KDEBundling.KDEBundling TheKDBBundling;

        GLControl ViewOriginalGraph;
        GLControl ViewBundledGraph;

        Record[][] OriginalGraph;
        Record[][] BundledGraph;

        public KDEBundlingForm()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //controls setup
            ViewOriginalGraph = new GLControl();
            ViewOriginalGraph.Location = new System.Drawing.Point(12, 12);
            ViewOriginalGraph.Size = new System.Drawing.Size(300, 300);
            ViewOriginalGraph.Paint += new PaintEventHandler(ViewOriginalGraph_Paint);

            ViewBundledGraph = new GLControl();
            ViewBundledGraph.Location = new System.Drawing.Point(318, 12);

            ViewBundledGraph.Size = new System.Drawing.Size(300, 300);
            ViewBundledGraph.Paint += new PaintEventHandler(ViewBundledGraph_Paint);

            this.Controls.Add(ViewOriginalGraph);
            this.Controls.Add(ViewBundledGraph);

            OriginalGraph = CreateTestDataset();

            //Do the bundling
            TheKDBBundling = new KDEBundling.KDEBundling();

            TheKDBBundling.SetData(OriginalGraph);
            TheKDBBundling.DoBundling();

            BundledGraph = TheKDBBundling.RecordsCurrent;
        }

        private Record[][] CreateTestDataset()
        {
            int nbrLines = 10;
            int nbrPointsPerLine = 10;

            Record[][] rec = new Record[nbrLines][];

            for (int i = 0; i < nbrLines; i++)
            {
                rec[i] = new Record[nbrPointsPerLine];

                for (int j = 0; j < nbrPointsPerLine; j++)
                {
                    double incy = 1.0 / nbrPointsPerLine;
                    double incx = (1.0 - 2 * (double)i / (double)nbrLines) / (double)nbrPointsPerLine;
                    rec[i][j] = new Record((double)i / (double)nbrLines + incx * j, incy * j, 1.0f);
                }
            }
            return rec;
        }      

        #region Drawing

        void ViewOriginalGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(ViewOriginalGraph, OriginalGraph);
        }

        private void DrawGraph(GLControl view, Record[][] dataset)
        {         
            view.MakeCurrent();

            GL.Viewport(0, 0, ViewBundledGraph.Width, ViewBundledGraph.Height);
            GL.ClearColor(Color.LightGray);
            GL.Clear(ClearBufferMask.ColorBufferBit);

            GL.Color3(Color.Black);
            
            foreach (var ns in dataset)
            {
                GL.Begin(BeginMode.LineStrip);
                List<PointF> pts = new List<PointF>();
                foreach (var n in ns)
                {
                    GL.Vertex2(n.X * 2.0f - 1.0f, n.Y * 2.0f - 1.0f);
                }
                GL.End();
            }

            view.SwapBuffers();
        }


        void ViewBundledGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(ViewBundledGraph, BundledGraph);      
        }

        #endregion
 

    }
}
